import os, re, json, random, time, string
from ollama import Client
import chromadb
from langchain_ollama import ChatOllama


class OllamaGenerator:

    def __init__(self, title: str, content: str, inner_title: str):
        self.title = title
        self.inner_title = inner_title
        self.content = content
        self.response = None
        self.chroma = chromadb.HttpClient(host="172.18.0.2", port=8000)
        ollama_url = f"{os.environ["OLLAMA_PROTOCOL"]}://{os.environ["OLLAMA_HOST"]}:{os.environ["OLLAMA_PORT"]}"
        self.ollama_client = Client(host=ollama_url)
        self.ollama_model = os.environ["EDITOR_MODEL"]
        self.embed_model = os.environ["EMBEDDING_MODEL"]
        self.agent_models = json.loads(os.environ["CONTENT_CREATOR_MODELS"])
        self.llm = ChatOllama(model=self.ollama_model, temperature=0.6, top_p=0.5) #This  is the level head in the room
        self.prompt_inject = f"""
            You are a journalist, Software Developer and DevOps expert
            writing a 1000 word draft blog for other tech enthusiasts.
            You like to use almost no code examples and prefer to talk
            in a light comedic tone. You are also Australian
            As this person write this blog as a markdown document.
            The title for the blog is {self.inner_title}.
            Do not output the title in the markdown.
            The basis for the content of the blog is:
                {self.content}
            """

    def split_into_chunks(self, text, chunk_size=100):
        '''Split text into chunks of size chunk_size'''
        words = re.findall(r'\S+', text)

        chunks = []
        current_chunk = []
        word_count = 0

        for word in words:
            current_chunk.append(word)
            word_count += 1

        if word_count >= chunk_size:
            chunks.append(' '.join(current_chunk))
            current_chunk = []
            word_count = 0

        if current_chunk:
            chunks.append(' '.join(current_chunk))

        return chunks

    def generate_draft(self, model) -> str:
        '''Generate a draft blog post using the specified model'''
        try:
            # the idea behind this is to make the "creativity" random amongst the content creators
            # contorlling temperature will allow cause the output to allow more "random" connections in sentences
            # Controlling top_p will tighten or loosen the embedding connections made
            # The result should be varied levels of "creativity" in the writing of the drafts
            # for more see https://python.langchain.com/v0.2/api_reference/ollama/chat_models/langchain_ollama.chat_models.ChatOllama.html
            temp = random.uniform(0.5, 1.0)
            top_p = random.uniform(0.4, 0.8)
            top_k = int(random.uniform(30, 80))
            agent_llm = ChatOllama(model=model, temperature=temp, top_p=top_p, top_k=top_k)
            messages = [
                ("system", self.prompt_inject),
                ("human", "make the blog post in a format to be edited easily" )
            ]
            response = agent_llm.invoke(messages)
            # self.response = self.ollama_client.chat(model=model,
            #                                         messages=[
            #         {
            #             'role': 'user',
            #             'content': f'{self.prompt_inject}',
            #         },
            #     ])
            #print ("draft")
            #print (response)
            return response.text()#['message']['content']

        except Exception as e:
            raise Exception(f"Failed to generate blog draft: {e}")

    def get_draft_embeddings(self, draft_chunks):
        '''Get embeddings for the draft chunks'''
        embeds = self.ollama_client.embed(model=self.embed_model, input=draft_chunks)
        return embeds.get('embeddings', [])

    def id_generator(self, size=6, chars=string.ascii_uppercase + string.digits):
        return ''.join(random.choice(chars) for _ in range(size))

    def load_to_vector_db(self):
        '''Load the generated blog drafts into a vector database'''
        collection_name = f"blog_{self.title.lower().replace(" ", "_")}_{self.id_generator()}"
        collection = self.chroma.get_or_create_collection(name=collection_name)#, metadata={"hnsw:space": "cosine"})
        #if any(collection.name == collectionname for collectionname in self.chroma.list_collections()):
        #    self.chroma.delete_collection("blog_creator")
        for model in self.agent_models:
            print (f"Generating draft from {model} for load into vector database")
            draft_chunks = self.split_into_chunks(self.generate_draft(model))
            print(f"generating embeds")
            embeds = self.get_draft_embeddings(draft_chunks)
            ids = [model + str(i) for i in range(len(draft_chunks))]
            chunknumber = list(range(len(draft_chunks)))
            metadata = [{"model_agent": model} for index in chunknumber]
            print(f'loading into collection')
            collection.add(documents=draft_chunks, embeddings=embeds, ids=ids, metadatas=metadata)

        return collection


    def generate_markdown(self) -> str:

        prompt_system = f"""
            You are an editor taking information from {len(self.agent_models)} Software
            Developers and Data experts
            writing a 3000 word blog for other tech enthusiasts.
            You like when they use almost no code examples and the
            voice is in a light comedic tone. You are also Australian
            As this person produce and an amalgamtion of this blog as a markdown document.
            The title for the blog is {self.inner_title}.
            Do not output the title in the markdown. Avoid repeated sentences
            The basis for the content of the blog is:
                {self.content}
            """
        try:
<<<<<<< HEAD
            query_embed = self.ollama_client.embed(model=self.embed_model, input=prompt_system)['embeddings']
            collection = self.load_to_vector_db()
            collection_query = collection.query(query_embeddings=query_embed, n_results=100)
            print("Showing pertinent info from drafts used in final edited edition")
            pertinent_draft_info = '\n\n'.join(collection.query(query_embeddings=query_embed, n_results=100)['documents'][0])
            #print(pertinent_draft_info)
            prompt_human = f"Generate the final document using this information from the drafts: {pertinent_draft_info} - ONLY OUTPUT THE MARKDOWN"
            print("Generating final document")
            messages = [("system", prompt_system), ("human", prompt_human),]
            self.response = self.llm.invoke(messages).text()
            # self.response = self.ollama_client.chat(model=self.ollama_model,
            #                                         messages=[
            #         {
            #             'role': 'user',
            #             'content': f'{prompt_enhanced}',
            #         },
            #     ])
            #print ("Markdown Generated")
            #print (self.response)
            return self.response#['message']['content']
=======
            self.response = self.ollama_client.chat(model=self.ollama_model,
                                                    messages=[
                    {
                        'role': 'user',
                        'content': f'{prompt}',
                    },
                ])

            # the deepseek model returns <think> this removes those tabs from the output
            return re.sub(r"<think|.\n\r+?|([^;]*)\/think>",'',self.response['message']['content'])
>>>>>>> e1a24af (get rid of think tags)

        except Exception as e:
            raise Exception(f"Failed to generate markdown: {e}")

    def save_to_file(self, filename: str) -> None:
        with open(filename, "w") as f:
            f.write(self.generate_markdown())

<<<<<<< HEAD
    def generate_commit_message(self):
        prompt_system = "You are a blog creator commiting a piece of content to a central git repo"
        prompt_human = f"Generate a 5 word git commit message describing {self.response}"
        messages = [("system", prompt_system), ("human", prompt_human),]
        commit_message = self.llm.invoke(messages).text()
        return commit_message
=======
>>>>>>> e1a24af (get rid of think tags)
